@@ -144,6 +144,7 @@ struct headset {
 	gboolean cli_active;
 	gboolean cme_enabled;
 	gboolean cwa_enabled;
+	gboolean pending_ring;
 
 	headset_state_t state;
 	struct pending_connect *pending;
@@ -457,6 +458,55 @@ static unsigned int connect_cb_new(struct headset *hs,
 	return cb->id;
 }
 
+static void send_foreach_headset(GSList *devices,
+					int (*cmp) (struct headset *hs),
+					char *format, ...)
+{
+	GSList *l;
+	va_list ap;
+
+	for (l = devices; l != NULL; l = l->next) {
+		struct audio_device *device = l->data;
+		struct headset *hs = device->headset;
+		int ret;
+
+		assert(hs != NULL);
+
+		if (cmp && cmp(hs) != 0)
+			continue;
+
+		va_start(ap, format);
+		ret = headset_send_valist(hs, format, ap);
+		if (ret < 0)
+			error("Failed to send to headset: %s (%d)",
+					strerror(-ret), -ret);
+		va_end(ap);
+	}
+}
+
+static int cli_cmp(struct headset *hs)
+{
+	if (!hs->hfp_active)
+		return -1;
+
+	if (hs->cli_active)
+		return 0;
+	else
+		return -1;
+}
+
+static gboolean ring_timer_cb(gpointer data)
+{
+	send_foreach_headset(active_devices, NULL, "\r\nRING\r\n");
+
+	if (ag.number)
+		send_foreach_headset(active_devices, cli_cmp,
+					"\r\n+CLIP:\"%s\",%d\r\n",
+					ag.number, ag.number_type);
+
+	return TRUE;
+}
+
 static void sco_connect_cb(GIOChannel *chan, int err, const bdaddr_t *src,
 			const bdaddr_t *dst, gpointer user_data)
 {
@@ -490,8 +540,7 @@ static void sco_connect_cb(GIOChannel *chan, int err, const bdaddr_t *src,
 
 	if (p->msg) {
 		DBusMessage *reply = dbus_message_new_method_return(p->msg);
-		dbus_connection_send(dev->conn, reply, NULL);
-		dbus_message_unref(reply);
+		g_dbus_send_message(dev->conn, reply);
 	}
 
 	pending_connect_finalize(dev);
@@ -499,6 +548,13 @@ static void sco_connect_cb(GIOChannel *chan, int err, const bdaddr_t *src,
 	fcntl(sk, F_SETFL, 0);
 
 	headset_set_state(dev, HEADSET_STATE_PLAYING);
+
+	if (hs->pending_ring) {
+		ring_timer_cb(NULL);
+		ag.ring_timer = g_timeout_add(RING_INTERVAL, ring_timer_cb,
+						NULL);
+		hs->pending_ring = FALSE;
+	}
 }
 
 static int sco_connect(struct audio_device *dev, headset_stream_cb_t cb,
@@ -538,33 +594,6 @@ static int hfp_cmp(struct headset *hs)
 		return -1;
 }
 
-static void send_foreach_headset(GSList *devices,
-					int (*cmp)(struct headset *hs),
-					char *format, ...)
-{
-	GSList *l;
-	va_list ap;
-
-	for (l = devices; l != NULL; l = l->next) {
-		struct audio_device *device = l->data;
-		struct headset *hs = device->headset;
-		int ret;
-
-		assert(hs != NULL);
-
-		if (cmp && cmp(hs) != 0)
-			continue;
-
-		va_start(ap, format);
-		ret = headset_send_valist(hs, format, ap);
-		if (ret < 0)
-			error("Failed to send to headset: %s (%d)",
-					strerror(-ret), -ret);
-		va_end(ap);
-	}
-}
-
-
 static void hfp_slc_complete(struct audio_device *dev)
 {
 	struct headset *hs = dev->headset;
@@ -580,8 +609,7 @@ static void hfp_slc_complete(struct audio_device *dev)
 	if (p->target_state == HEADSET_STATE_CONNECTED) {
 		if (p->msg) {
 			DBusMessage *reply = dbus_message_new_method_return(p->msg);
-			dbus_connection_send(dev->conn, reply, NULL);
-			dbus_message_unref(reply);
+			g_dbus_send_message(dev->conn, reply);
 		}
 		pending_connect_finalize(dev);
 		return;
@@ -714,12 +742,11 @@ static int button_press(struct audio_device *device, const char *buf)
 
 int telephony_answer_call_rsp(void *telephony_device, cme_error_t err)
 {
-	struct audio_device *device = telephony_device;
-	struct headset *hs = device->headset;
-
-	if (err != CME_ERROR_NONE)
-		return telephony_generic_rsp(telephony_device, err);
+	return telephony_generic_rsp(telephony_device, err);
+}
 
+static int answer_call(struct audio_device *device, const char *buf)
+{
 	if (ag.ring_timer) {
 		g_source_remove(ag.ring_timer);
 		ag.ring_timer = 0;
@@ -730,16 +757,6 @@ int telephony_answer_call_rsp(void *telephony_device, cme_error_t err)
 		ag.number = NULL;
 	}
 
-	return headset_send(hs, "\r\nOK\r\n");
-}
-
-int telephony_anser_call_rsp(void *telephony_device, cme_error_t err)
-{
-	return telephony_generic_rsp(telephony_device, err);
-}
-
-static int answer_call(struct audio_device *device, const char *buf)
-{
 	telephony_answer_call_req(device);
 
 	return 0;
@@ -758,6 +775,11 @@ int telephony_terminate_call_rsp(void *telephony_device,
 			AUDIO_HEADSET_INTERFACE, "CallTerminated",
 			DBUS_TYPE_INVALID);
 
+	return headset_send(hs, "\r\nOK\r\n");
+}
+
+static int terminate_call(struct audio_device *device, const char *buf)
+{
 	if (ag.number) {
 		g_free(ag.number);
 		ag.number = NULL;
@@ -768,11 +790,6 @@ int telephony_terminate_call_rsp(void *telephony_device,
 		ag.ring_timer = 0;
 	}
 
-	return headset_send(hs, "\r\nOK\r\n");
-}
-
-static int terminate_call(struct audio_device *device, const char *buf)
-{
 	telephony_terminate_call_req(device);
 
 	return 0;
@@ -1197,8 +1214,7 @@ static void rfcomm_connect_cb(GIOChannel *chan, int err, const bdaddr_t *src,
 
 	if (p->msg) {
 		DBusMessage *reply = dbus_message_new_method_return(p->msg);
-		dbus_connection_send(dev->conn, reply, NULL);
-		dbus_message_unref(reply);
+		g_dbus_send_message(dev->conn, reply);
 	}
 
 	pending_connect_finalize(dev);
@@ -1466,6 +1482,10 @@ static DBusMessage *hs_connect(DBusConnection *conn, DBusMessage *msg,
 		return g_dbus_create_error(msg, ERROR_INTERFACE ".NotReady",
 					"Telephony subsystem not ready");
 
+	if (!manager_allow_headset_connection(&device->src))
+		return g_dbus_create_error(msg, ERROR_INTERFACE ".NotAllowed",
+						"Too many connected devices");
+
 	err = rfcomm_connect(device, NULL, NULL, NULL);
 	if (err < 0)
 		return g_dbus_create_error(msg, ERROR_INTERFACE
@@ -1479,29 +1499,6 @@ static DBusMessage *hs_connect(DBusConnection *conn, DBusMessage *msg,
 	return NULL;
 }
 
-static int cli_cmp(struct headset *hs)
-{
-	if (!hs->hfp_active)
-		return -1;
-
-	if (hs->cli_active)
-		return 0;
-	else
-		return -1;
-}
-
-static gboolean ring_timer_cb(gpointer data)
-{
-	send_foreach_headset(active_devices, NULL, "\r\nRING\r\n");
-
-	if (ag.number)
-		send_foreach_headset(active_devices, cli_cmp,
-					"\r\n+CLIP:\"%s\",%d\r\n",
-					ag.number, ag.number_type);
-
-	return TRUE;
-}
-
 static DBusMessage *hs_ring(DBusConnection *conn, DBusMessage *msg,
 					void *data)
 {
@@ -1585,6 +1582,10 @@ static DBusMessage *hs_play(DBusConnection *conn, DBusMessage *msg,
 						".NotConnected",
 						"Device not Connected");
 	case HEADSET_STATE_PLAY_IN_PROGRESS:
+		if (hs->pending && hs->pending->msg == NULL) {
+			hs->pending->msg = dbus_message_ref(msg);
+			return NULL;
+		}
 		return g_dbus_create_error(msg, ERROR_INTERFACE
 						".InProgress",
 						"Play in Progress");
@@ -2036,8 +2037,7 @@ uint32_t headset_config_init(GKeyFile *config)
 					&err);
 	if (err) {
 		debug("audio.conf: %s", err->message);
-		g_error_free(err);
-		err = NULL;
+		g_clear_error(&err);
 	} else {
 		if (strcmp(str, "PCM") == 0)
 			sco_hci = FALSE;
@@ -2169,6 +2169,13 @@ int headset_connect_sco(struct audio_device *dev, GIOChannel *io)
 
 	hs->sco = io;
 
+	if (hs->pending_ring) {
+		ring_timer_cb(NULL);
+		ag.ring_timer = g_timeout_add(RING_INTERVAL, ring_timer_cb,
+						NULL);
+		hs->pending_ring = FALSE;
+	}
+
 	return 0;
 }
 
@@ -2413,9 +2420,16 @@ int telephony_response_and_hold_ind(int rh)
 
 int telephony_incoming_call_ind(const char *number, int type)
 {
+	struct audio_device *dev;
+	struct headset *hs;
+
 	if (!active_devices)
 		return -ENODEV;
 
+	/* Get the latest connected device */
+	dev = active_devices->data;
+	hs = dev->headset;
+
 	if (ag.ring_timer) {
 		debug("telephony_incoming_call_ind: already calling");
 		return -EBUSY;
@@ -2425,6 +2439,21 @@ int telephony_incoming_call_ind(const char *number, int type)
 	ag.number = g_strdup(number);
 	ag.number_type = type;
 
+	if (ag.features & AG_FEATURE_INBAND_RINGTONE && hs->hfp_active &&
+			hs->state != HEADSET_STATE_PLAYING) {
+		if (hs->state == HEADSET_STATE_CONNECTED) {
+			int ret;
+
+			ret = sco_connect(dev, NULL, NULL, NULL);
+			if (ret < 0)
+				return ret;
+		}
+
+		hs->pending_ring = TRUE;
+
+		return 0;
+	}
+
 	ring_timer_cb(NULL);
 	ag.ring_timer = g_timeout_add(RING_INTERVAL, ring_timer_cb, NULL);
 
@@ -2433,14 +2462,23 @@ int telephony_incoming_call_ind(const char *number, int type)
 
 int telephony_calling_stopped_ind(void)
 {
+	struct audio_device *dev;
+
 	if (!active_devices)
 		return -ENODEV;
 
-	if (!ag.ring_timer)
+	/* In case SCO isn't fully up yet */
+	dev = active_devices->data;
+
+	if (!dev->headset->pending_ring && !ag.ring_timer)
 		return -EINVAL;
 
-	g_source_remove(ag.ring_timer);
-	ag.ring_timer = 0;
+	dev->headset->pending_ring = FALSE;
+
+	if (ag.ring_timer) {
+		g_source_remove(ag.ring_timer);
+		ag.ring_timer = 0;
+	}
 
 	return 0;
 }
