@@ -23,6 +23,7 @@
  */
 
 struct enabled_interfaces {
+	gboolean hfp;
 	gboolean headset;
 	gboolean gateway;
 	gboolean sink;
@@ -30,8 +31,6 @@ struct enabled_interfaces {
 	gboolean control;
 };
 
-typedef void (*create_dev_cb_t) (struct audio_device *dev, void *user_data);
-
 int audio_manager_init(DBusConnection *conn, GKeyFile *config);
 void audio_manager_exit(void);
 
@@ -40,5 +39,7 @@ gboolean server_is_enabled(bdaddr_t *src, uint16_t svc);
 struct audio_device *manager_find_device(const bdaddr_t *bda, const char *interface,
 					gboolean connected);
 
-gboolean manager_create_device(bdaddr_t *bda, create_dev_cb_t cb,
-				void *user_data);
+struct audio_device *manager_get_device(const bdaddr_t *src,
+					const bdaddr_t *dst);
+
+gboolean manager_allow_headset_connection(bdaddr_t *src);
