@@ -331,6 +331,12 @@ struct pending_auth_info *adapter_new_auth_request(struct btd_adapter *adapter,
 	return info;
 }
 
+static void dev_info_free(struct remote_dev_info *dev)
+{
+	g_free(dev->alias);
+	g_free(dev);
+}
+
 int pending_remote_name_cancel(struct btd_adapter *adapter)
 {
 	struct remote_dev_info *dev, match;
@@ -360,7 +366,7 @@ int pending_remote_name_cancel(struct btd_adapter *adapter)
 	}
 
 	/* free discovered devices list */
-	g_slist_foreach(adapter->found_devices, (GFunc) g_free, NULL);
+	g_slist_foreach(adapter->found_devices, (GFunc) dev_info_free, NULL);
 	g_slist_free(adapter->found_devices);
 	adapter->found_devices = NULL;
 
@@ -2637,7 +2643,8 @@ int adapter_stop(struct btd_adapter *adapter)
 	}
 
 	if (adapter->found_devices) {
-		g_slist_foreach(adapter->found_devices, (GFunc) g_free, NULL);
+		g_slist_foreach(adapter->found_devices,
+				(GFunc) dev_info_free, NULL);
 		g_slist_free(adapter->found_devices);
 		adapter->found_devices = NULL;
 	}
@@ -2757,11 +2764,12 @@ struct btd_adapter *adapter_create(DBusConnection *conn, int id)
 {
 	char path[MAX_PATH_LENGTH];
 	struct btd_adapter *adapter;
+	const char *base_path = manager_get_base_path();
 
 	if (!connection)
 		connection = conn;
 
-	snprintf(path, sizeof(path), "%s/hci%d", "/org/bluez", id);
+	snprintf(path, sizeof(path), "%s/hci%d", base_path, id);
 
 	adapter = g_try_new0(struct btd_adapter, 1);
 	if (!adapter) {
@@ -2936,7 +2944,8 @@ void adapter_set_state(struct btd_adapter *adapter, int state)
 				(GSourceFunc) start_inquiry, adapter);
 
 	if (!discov_active && adapter->found_devices) {
-		g_slist_foreach(adapter->found_devices, (GFunc) g_free, NULL);
+		g_slist_foreach(adapter->found_devices,
+					(GFunc) dev_info_free, NULL);
 		g_slist_free(adapter->found_devices);
 		adapter->found_devices = NULL;
 	}
@@ -2983,7 +2992,8 @@ int dev_rssi_cmp(struct remote_dev_info *d1, struct remote_dev_info *d2)
 }
 
 int adapter_add_found_device(struct btd_adapter *adapter, bdaddr_t *bdaddr,
-				int8_t rssi, name_status_t name_status)
+				int8_t rssi, uint32_t class, const char *alias,
+				name_status_t name_status)
 {
 	struct remote_dev_info *dev, match;
 
@@ -2998,6 +3008,13 @@ int adapter_add_found_device(struct btd_adapter *adapter, bdaddr_t *bdaddr,
 		if (rssi != 0)
 			dev->rssi = rssi;
 
+		dev->class = class;
+
+		if (alias) {
+			g_free(dev->alias);
+			dev->alias = g_strdup(alias);
+		}
+
 		 /* Get remote name can be received while inquiring.
 		  * Keep in mind that multiple inquiry result events can
 		  * be received from the same remote device.
@@ -3015,6 +3032,9 @@ int adapter_add_found_device(struct btd_adapter *adapter, bdaddr_t *bdaddr,
 
 	bacpy(&dev->bdaddr, bdaddr);
 	dev->rssi = rssi;
+	dev->class = class;
+	if (alias)
+		dev->alias = g_strdup(alias);
 	dev->name_status = name_status;
 
 	adapter->found_devices = g_slist_insert_sorted(adapter->found_devices,
@@ -3035,7 +3055,7 @@ int adapter_remove_found_device(struct btd_adapter *adapter, bdaddr_t *bdaddr)
 		return -1;
 
 	adapter->found_devices = g_slist_remove(adapter->found_devices, dev);
-	g_free(dev);
+	dev_info_free(dev);
 
 	return 0;
 }
@@ -3178,7 +3198,7 @@ struct active_conn_info *adapter_search_active_conn_by_bdaddr(struct btd_adapter
 	if (!adapter || !adapter->active_conn)
 		return NULL;
 
-	l = g_slist_find_custom(adapter->active_conn, &bda,
+	l = g_slist_find_custom(adapter->active_conn, bda,
 					active_conn_find_by_bdaddr);
 	if (l)
 		return l->data;
@@ -3273,31 +3293,23 @@ static void agent_auth_cb(struct agent *agent, DBusError *derr, void *user_data)
 	g_free(auth);
 }
 
-int btd_request_authorization(const bdaddr_t *src, const bdaddr_t *dst,
+static int btd_adapter_authorize(struct btd_adapter *adapter, const bdaddr_t *dst,
 		const char *uuid, service_auth_cb cb, void *user_data)
 {
 	struct service_auth *auth;
-	struct btd_adapter *adapter;
 	struct btd_device *device;
 	struct agent *agent;
 	char address[18];
 	gboolean trusted;
 	const gchar *dev_path;
 
-	if (src == NULL || dst == NULL)
-		return -EINVAL;
-
-	adapter = manager_find_adapter(src);
-	if (!adapter)
-		return -EPERM;
-
 	/* Device connected? */
 	if (!g_slist_find_custom(adapter->active_conn,
 				dst, active_conn_find_by_bdaddr))
 		return -ENOTCONN;
 
 	ba2str(dst, address);
-	trusted = read_trust(src, address, GLOBAL_TRUST);
+	trusted = read_trust(&adapter->bdaddr, address, GLOBAL_TRUST);
 
 	if (trusted) {
 		cb(NULL, user_data);
@@ -3328,6 +3340,40 @@ int btd_request_authorization(const bdaddr_t *src, const bdaddr_t *dst,
 	return agent_authorize(agent, dev_path, uuid, agent_auth_cb, auth);
 }
 
+int btd_request_authorization(const bdaddr_t *src, const bdaddr_t *dst,
+		const char *uuid, service_auth_cb cb, void *user_data)
+{
+	struct btd_adapter *adapter;
+	GSList *adapters;
+
+	if (src == NULL || dst == NULL)
+		return -EINVAL;
+
+	if (bacmp(src, BDADDR_ANY) != 0)
+		goto proceed;
+
+	/* Handle request authorization for ANY adapter */
+	adapters = manager_get_adapters();
+
+	for (; adapters; adapters = adapters->next) {
+		int err;
+		adapter = adapters->data;
+
+		err = btd_adapter_authorize(adapter, dst, uuid, cb, user_data);
+		if (err == 0)
+			return 0;
+	}
+
+	return -EPERM;
+
+proceed:
+	adapter = manager_find_adapter(src);
+	if (!adapter)
+		return -EPERM;
+
+	return btd_adapter_authorize(adapter, dst, uuid, cb, user_data);
+}
+
 int btd_cancel_authorization(const bdaddr_t *src, const bdaddr_t *dst)
 {
 	struct btd_adapter *adapter = manager_find_adapter(src);
@@ -3377,3 +3423,33 @@ void adapter_update_devices(struct btd_adapter *adapter)
 					DBUS_TYPE_OBJECT_PATH, &devices);
 	g_free(devices);
 }
+
+static gchar *adapter_any_path = NULL;
+static int adapter_any_refcount = 0;
+
+const char *adapter_any_get_path(void)
+{
+	return adapter_any_path;
+}
+
+const char *btd_adapter_any_request_path(void)
+{
+	if (adapter_any_refcount > 0)
+		return adapter_any_path;
+
+	adapter_any_path = g_strdup_printf("%s/any", manager_get_base_path());
+	adapter_any_refcount++;
+
+	return adapter_any_path;
+}
+
+void btd_adapter_any_release_path(void)
+{
+	adapter_any_refcount--;
+
+	if (adapter_any_refcount > 0)
+		return;
+
+	g_free(adapter_any_path);
+	adapter_any_path = NULL;
+}
