@@ -450,6 +450,9 @@ int hcid_dbus_user_confirm(bdaddr_t *sba, bdaddr_t *dba, uint32_t passkey)
 
 		hci_close_dev(dd);
 
+		auth = adapter_new_auth_request(adapter, dba, AUTH_TYPE_CONFIRM);
+		auth->replied = TRUE;
+
 		return 0;
 	}
 
@@ -608,7 +611,8 @@ void hcid_dbus_bonding_process_complete(bdaddr_t *local, bdaddr_t *peer,
 	/* If this is a new pairing send the appropriate signal for it
 	 * and proceed with service discovery */
 	if (status == 0) {
-		if (device_set_paired(connection, device, bonding) && bonding)
+		device_set_paired(connection, device, bonding);
+		if (bonding)
 			adapter_free_bonding_request(adapter);
 		return;
 	}
@@ -845,7 +849,7 @@ static void append_dict_valist(DBusMessageIter *iter,
 	while (key) {
 		type = va_arg(var_args, int);
 		val = va_arg(var_args, void *);
-		dbus_message_iter_append_dict_entry(&dict, key, type, val);
+		dict_append_entry(&dict, key, type, val);
 		key = va_arg(var_args, char *);
 	}
 
@@ -873,9 +877,7 @@ static void emit_device_found(const char *path, const char *address,
 	append_dict_valist(&iter, first_key, var_args);
 	va_end(var_args);
 
-	dbus_connection_send(connection, signal, NULL);
-
-	dbus_message_unref(signal);
+	g_dbus_send_message(connection, signal);
 }
 
 void hcid_dbus_inquiry_result(bdaddr_t *local, bdaddr_t *peer, uint32_t class,
@@ -884,6 +886,7 @@ void hcid_dbus_inquiry_result(bdaddr_t *local, bdaddr_t *peer, uint32_t class,
 	char filename[PATH_MAX + 1];
 	struct btd_adapter *adapter;
 	char local_addr[18], peer_addr[18], *alias, *name, *tmp_name;
+	const char *real_alias;
 	const char *path, *icon, *paddr = peer_addr;
 	struct remote_dev_info *dev, match;
 	dbus_int16_t tmp_rssi = rssi;
@@ -940,12 +943,15 @@ void hcid_dbus_inquiry_result(bdaddr_t *local, bdaddr_t *peer, uint32_t class,
 	name = textfile_get(filename, peer_addr);
 
 	if (!alias) {
+		real_alias = NULL;
+
 		if (!name) {
 			alias = g_strdup(peer_addr);
 			g_strdelimit(alias, ":", '-');
 		} else
 			alias = g_strdup(name);
-	}
+	} else
+		real_alias = alias;
 
 	tmp_name = extract_eir_name(data, &name_type);
 	if (tmp_name) {
@@ -968,11 +974,7 @@ void hcid_dbus_inquiry_result(bdaddr_t *local, bdaddr_t *peer, uint32_t class,
 	path = adapter_get_path(adapter);
 	icon = class_to_icon(class);
 
-	if (name) {
-		if (name_type != 0x08)
-			name_status = NAME_SENT;
-
-		emit_device_found(path, paddr,
+	emit_device_found(path, paddr,
 				"Address", DBUS_TYPE_STRING, &paddr,
 				"Class", DBUS_TYPE_UINT32, &class,
 				"Icon", DBUS_TYPE_STRING, &icon,
@@ -980,20 +982,15 @@ void hcid_dbus_inquiry_result(bdaddr_t *local, bdaddr_t *peer, uint32_t class,
 				"Name", DBUS_TYPE_STRING, &name,
 				"Alias", DBUS_TYPE_STRING, &alias, NULL);
 
-		g_free(name);
-	} else {
-		emit_device_found(path, paddr,
-				"Address", DBUS_TYPE_STRING, &paddr,
-				"Class", DBUS_TYPE_UINT32, &class,
-				"Icon", DBUS_TYPE_STRING, &icon,
-				"RSSI", DBUS_TYPE_INT16, &tmp_rssi,
-				"Alias", DBUS_TYPE_STRING, &alias, NULL);
-	}
-
-	g_free(alias);
+	if (name && name_type != 0x08)
+		name_status = NAME_SENT;
 
 	/* add in the list to track name sent/pending */
-	adapter_add_found_device(adapter, peer, rssi, name_status);
+	adapter_add_found_device(adapter, peer, rssi, class, real_alias,
+					name_status);
+
+	g_free(name);
+	g_free(alias);
 }
 
 void hcid_dbus_remote_class(bdaddr_t *local, bdaddr_t *peer, uint32_t class)
@@ -1024,8 +1021,7 @@ void hcid_dbus_remote_class(bdaddr_t *local, bdaddr_t *peer, uint32_t class)
 
 	dev_path = device_get_path(device);
 
-	dbus_connection_emit_property_changed(connection, dev_path,
-				DEVICE_INTERFACE, "Class",
+	emit_property_changed(connection, dev_path, DEVICE_INTERFACE, "Class",
 				DBUS_TYPE_UINT32, &class);
 }
 
@@ -1034,8 +1030,9 @@ void hcid_dbus_remote_name(bdaddr_t *local, bdaddr_t *peer, uint8_t status,
 {
 	struct btd_adapter *adapter;
 	char srcaddr[18], dstaddr[18];
-	const gchar *dev_path;
 	int state;
+	struct btd_device *device;
+	struct remote_dev_info match, *dev_info;
 
 	adapter = manager_find_adapter(local);
 	if (!adapter) {
@@ -1046,29 +1043,51 @@ void hcid_dbus_remote_name(bdaddr_t *local, bdaddr_t *peer, uint8_t status,
 	ba2str(local, srcaddr);
 	ba2str(peer, dstaddr);
 
-	if (!status) {
-		struct btd_device *device;
+	if (status != 0)
+		goto proceed;
 
-		device = adapter_find_device(adapter, dstaddr);
-		if (device) {
-			char alias[248];
+	bacpy(&match.bdaddr, peer);
+	match.name_status = NAME_ANY;
 
-			dev_path = device_get_path(device);
+	dev_info = adapter_search_found_devices(adapter, &match);
+	if (dev_info) {
+		const char *adapter_path = adapter_get_path(adapter);
+		const char *icon = class_to_icon(dev_info->class);
+		const char *alias, *paddr = dstaddr;
+		dbus_int16_t rssi = dev_info->rssi;
 
-			dbus_connection_emit_property_changed(connection,
-						dev_path, DEVICE_INTERFACE,
-						"Name", DBUS_TYPE_STRING, &name);
+		if (dev_info->alias)
+			alias = dev_info->alias;
+		else
+			alias = name;
 
-			if (read_device_alias(srcaddr, dstaddr,
-						alias, sizeof(alias)) < 1) {
-
-				dbus_connection_emit_property_changed(connection,
-						dev_path, DEVICE_INTERFACE,
-						"Alias", DBUS_TYPE_STRING, &name);
-			}
-		}
+		emit_device_found(adapter_path, dstaddr,
+				"Address", DBUS_TYPE_STRING, &paddr,
+				"Class", DBUS_TYPE_UINT32, &dev_info->class,
+				"Icon", DBUS_TYPE_STRING, &icon,
+				"RSSI", DBUS_TYPE_INT16, &rssi,
+				"Name", DBUS_TYPE_STRING, &name,
+				"Alias", DBUS_TYPE_STRING, &alias,
+				NULL);
 	}
 
+	device = adapter_find_device(adapter, dstaddr);
+	if (device) {
+		char alias[248];
+		const char *dev_path = device_get_path(device);
+
+		emit_property_changed(connection, dev_path,
+				DEVICE_INTERFACE, "Name",
+				DBUS_TYPE_STRING, &name);
+
+		if (read_device_alias(srcaddr, dstaddr,
+					alias, sizeof(alias)) < 1)
+			emit_property_changed(connection, dev_path,
+					DEVICE_INTERFACE, "Alias",
+					DBUS_TYPE_STRING, &name);
+	}
+
+proceed:
 	/* remove from remote name request list */
 	adapter_remove_found_device(adapter, peer);
 
@@ -1098,6 +1117,10 @@ void hcid_dbus_link_key_notify(bdaddr_t *local, bdaddr_t *peer)
 	ba2str(peer, peer_addr);
 
 	device = adapter_find_device(adapter, peer_addr);
+	if (!device) {
+		error("link_key_notify: device %s was removed!", peer_addr);
+		return;
+	}
 
 	bonding = adapter_get_bonding_info(adapter);
 
@@ -1203,8 +1226,7 @@ void hcid_dbus_disconn_complete(bdaddr_t *local, uint8_t status,
 		} else {
 			reply = new_authentication_return(bonding->msg,
 							HCI_AUTHENTICATION_FAILURE);
-			dbus_connection_send(connection, reply, NULL);
-			dbus_message_unref(reply);
+			g_dbus_send_message(connection, reply);
 		}
 		adapter_free_bonding_request(adapter);
 	}
